/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.net.*;
import org.globus.replica.rls.*;

public class RLSGui extends JPanel
  implements ActionListener, ItemListener {

  private JFrame frame;
  private RLSTree rlstree;
  private Console console;
  private JPopupMenu popup;
  private RLSServer selectedrls = null;
  private RLSServer menurls = null;
  private RLSUpdate rlsupdate;
  private static final int TW = 800;
  private static final int TH = 700;
  private static final int CH = TH / 5;
  private static final int BH = 100;
  private static final int LW = TW / 2;
  private static final int RLW = 30;
  private static final int RIW = TW - LW - RLW;
  private static final int TOPH = 340;

  public RLSGui(String args[]) {
    frame = new JFrame("Replica Location Service Demo");

    JMenuBar menubar = new JMenuBar();
    frame.setJMenuBar(menubar);
    JMenu menu = new JMenu("File");
    menubar.add(menu);
    JMenuItem menuitem = new JMenuItem("Quit");
    menuitem.addActionListener(this);
    menu.add(menuitem);
    menuitem = new JMenuItem("View Topology");
    menuitem.addActionListener(this);
    menu.add(menuitem);

    Container contents = frame.getContentPane();
    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.BOTH;
    contents.setLayout(gb);

    rlstree = new RLSTree(args);
    gbc.gridx = 0;
    gbc.gridy = 0;
    gbc.weightx = 0.5;
    gbc.weighty = 1.0;
    gb.setConstraints(rlstree, gbc);
    contents.add(rlstree);

    rlsupdate = new RLSUpdate();
    gbc.gridx = 1;
    gbc.gridy = 0;
    gbc.weightx = 0.5;
    gbc.weighty = 1.0;
    gb.setConstraints(rlsupdate, gbc);
    contents.add(rlsupdate);

    console = new Console();
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 2;
    gbc.weightx = 0.0;
    gbc.weighty = 1.0;
    gb.setConstraints(console, gbc);
    contents.add(console);

    popup = new JPopupMenu();
    JMenuItem item = new JMenuItem("Get LFNs");
    item.addActionListener(this);
    popup.add(item);
    item = new JMenuItem("Info");
    item.addActionListener(this);
    popup.add(item);
    item = new JMenuItem("Topology");
    item.addActionListener(this);
    popup.add(item);
    item = new JMenuItem("Update");
    item.addActionListener(this);
    popup.add(item);

    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });

    frame.pack();
    frame.setVisible(true);
  }

  class RLSUpdate extends JPanel implements ActionListener {
    JTextField lfn;
    JTextField pfn;
    ResultTree rt;
    int tfnum;

    public RLSUpdate() {
      tfnum = 0;
      setLayout(new BorderLayout());

      JPanel p = new JPanel();
      GridBagLayout gb = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();
      p.setLayout(gb);
      p.setSize(RIW, BH);

      JLabel label = new JLabel("LFN");
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(label, gbc);
      p.add(label);

      Font font = getFont();
      lfn = new JTextField(RIW / font.getSize());
      gbc.gridx = 1;
      gbc.gridy = 0;
      gbc.gridwidth = 3;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(lfn, gbc);
      p.add(lfn);

      label = new JLabel("PFN");
      gbc.gridx = 0;
      gbc.gridy = 1;
      gbc.gridwidth = 1;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(label, gbc);
      p.add(label);

      pfn = new JTextField(RIW / font.getSize());
      gbc.gridx = 1;
      gbc.gridy = 1;
      gbc.gridwidth = 3;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(pfn, gbc);
      p.add(pfn);

      gbc.gridwidth = 1;

      JButton jb = new JButton("create");
      jb.setActionCommand("create");
      jb.addActionListener(this);
      gbc.gridx = 0;
      gbc.gridy = 2;
      gb.setConstraints(jb, gbc);
      p.add(jb);

      jb = new JButton("delete");
      jb.setActionCommand("delete");
      jb.addActionListener(this);
      gbc.gridx = 1;
      gbc.gridy = 2;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(jb, gbc);
      p.add(jb);

      jb = new JButton("query");
      jb.setActionCommand("query");
      jb.addActionListener(this);
      gbc.gridx = 2;
      gbc.gridy = 2;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(jb, gbc);
      p.add(jb);

      jb = new JButton("query all");
      jb.setActionCommand("query all");
      jb.addActionListener(this);
      gbc.gridx = 3;
      gbc.gridy = 2;
      gbc.anchor = GridBagConstraints.WEST;
      gb.setConstraints(jb, gbc);
      p.add(jb);

      add(p, BorderLayout.NORTH);

      label = new JLabel("Query Results");
      add(label, BorderLayout.CENTER);

      rt = new ResultTree();
      add(rt, BorderLayout.SOUTH);
    }

    public void actionPerformed(ActionEvent e) {
      String tpfn;
      int r;

      String cmd = e.getActionCommand();
      if (selectedrls == null && !cmd.equals("query all")) {
	console.msg("No server selected");
	return;
      }
      String l = lfn.getText();
      String p = pfn.getText();

      try {
	boolean lrcres;

	if (cmd.equals("query")) {
	  ArrayList list;

	  if (rlstree.curnode.isNodeAncestor(rlstree.rlitree)) {
	    lrcres = false;
	    if (l.length() == 0) {
	      console.msg("No LFN specified");
	      return;
	    }
	    if ((selectedrls.s.Flags & RLSStats.RLS_RCVLFNLIST) != 0)
	      list = selectedrls.h.RLIGetLRCWC(l);
	    else
	      list = selectedrls.h.RLIGetLRC(l);
	  } else {
	    lrcres = true;
	    if (l.length() > 0)
	      list = selectedrls.h.LRCGetPFNWC(l);
	    else if (p.length() > 0)
	      list = selectedrls.h.LRCGetLFNWC(p);
	    else {
	      console.msg("Must specify LFN or PFN");
	      return;
	    }
	  }

	  rt.clear(lrcres);
	  Iterator iter = list.iterator();
	  while (iter.hasNext()) {
	    RLSString2 str2 = (RLSString2) iter.next();
	    rt.addnode(str2);
	  }
	} else if (cmd.equals("query all")) {
	  if (l.length() <= 0) {
	    console.msg("Must specify LFN");
	    return;
	  }
	  rt.clear(true);
	  Iterator lrci = rlstree.lrcs.iterator();
	  while (lrci.hasNext()) {
	    RLSServer server = (RLSServer) lrci.next();
	    try {
	      ArrayList list = server.h.LRCGetPFN(l);
	      Iterator iter = list.iterator();
	      while (iter.hasNext()) {
		RLSString2 str2 = (RLSString2) iter.next();
		rt.addnode(str2);
	      }
	    } catch (RLSException ex) {
	      if (ex.GetRC() != RLSClient.RLS_LFN_NEXIST)
		System.out.println(server.url + ": " + ex.toString());	      
	    }
	  }
	} else {
	  if ((selectedrls.s.Flags & RLSStats.RLS_LRCSERVER) == 0) {
	    console.msg(selectedrls.url + " is not an LRC server");
	    return;
	  }
	  if (l.length() == 0 || p.length() == 0) {
	    console.msg("Must specify both LFN and PFN");
	    return;
	  }
	  p = mkpfn(selectedrls, p);
	  if (cmd.equals("create")) {
	    tpfn = "/tmp/rlsgui." + tfnum++;
	    r = RLSClient.OSSystem("rm -f " + tpfn + "; emacs " + tpfn);
	    if (r != 0) {
	      console.msg("Unable to create " + tpfn + " status " + r);
	      return;
	    }
	    r = RLSClient.OSSystem("globus-url-copy file://" + tpfn + " " + p);
	    if (r != 0) {
	      console.msg("Unable to copy " + tpfn + " to " + p);
	      return;
	    }
	    selectedrls.h.LRCCreate(l, p);
	  } else if (cmd.equals("delete"))
	    selectedrls.h.LRCDelete(l, p);
	  console.msg(cmd + "(" + l + "," + p + ") successful");
	}
      } catch (RLSException ex) {
	console.msg(cmd + "(" + l + "," + p + "): " + ex);
      }
    }

}

  class RLSTree extends JPanel {
    DefaultMutableTreeNode root;
    DefaultTreeModel treemodel;
    DefaultMutableTreeNode rlitree;
    DefaultMutableTreeNode lrctree;
    JTree tree;
    RLSServer rlsserver[];
    DefaultMutableTreeNode curnode;
    ArrayList lrcs;
    ArrayList rlis;
    Cursor movecur;
    Cursor defcur;
    boolean dragging;
    String draglfn;
    String dragpfn;
    String tf;
    int tfnum = 0;

    public RLSTree(String args[]) {
      int i;

      movecur = new Cursor(Cursor.MOVE_CURSOR);
      defcur = new Cursor(Cursor.DEFAULT_CURSOR);
      tf = "/tmp/rlsgui." + System.getProperty("user.name");
      dragging = false;
      lrcs = new ArrayList();
      rlis = new ArrayList();
      rlsserver = new RLSServer[args.length];
      for (i = 0; i < args.length; i++)
	rlsserver[i] = new RLSServer(args[i]);
      for (i = 0; i < args.length; i++)
	if (rlsserver[i].h != null)
	  break;
      if (i >= args.length) {
	System.out.println("Unable to connect to any RLS servers");
	System.exit(0);
      }

      root = new DefaultMutableTreeNode("Servers");
      rlitree = new DefaultMutableTreeNode("RLI");
      lrctree = new DefaultMutableTreeNode("LRC");
      root.add(rlitree);
      root.add(lrctree);
      for (i = 0; i < rlsserver.length; i++) {
	if (rlsserver[i].h == null)
	  continue;
	if ((rlsserver[i].s.Flags & RLSStats.RLS_RLISERVER) != 0) {
	  rlitree.add(new DefaultMutableTreeNode(rlsserver[i]));
	  rlis.add(0, rlsserver[i]);
	}
	if ((rlsserver[i].s.Flags & RLSStats.RLS_LRCSERVER) != 0) {
	  lrctree.add(new DefaultMutableTreeNode(rlsserver[i]));
	  lrcs.add(0, rlsserver[i]);
	}
      }

      treemodel = new DefaultTreeModel(root);
      tree = new JTree(treemodel);
      tree.setEditable(true);
      tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
      tree.addMouseListener(new MouseAdapter() {
	public void mousePressed(MouseEvent me) {
	  treeMousePressed(me);
	}
	public void mouseReleased(MouseEvent me) {
	  treeMouseReleased(me);
	}
      });

      JScrollPane sp = new JScrollPane(tree);
      sp.setPreferredSize(new Dimension(LW,TH-CH));

      add(sp);
    }

    public void CloseAll() {
      for (int i = 0; i < rlsserver.length; i++)
	try {
	  if (rlsserver[i].h != null)
	    rlsserver[i].h.Close();
	} catch (RLSException e) {
	  console.msg(rlsserver[i].url + ": " + e);
	}
    }

    public void Update(RLSServer rls) {
      try {
	rls.h.Admin(RLSClient.ADMIN_SSU);
	console.msg("Requested " + rls.url + " send softstate updates");
      } catch (RLSException ex) {
	console.msg(rls.url + ": " + ex);
      }
    }

    public void GetLFNs(RLSServer rls) {
      curnode.removeAllChildren();
      treemodel.reload(curnode);
      try {
	ArrayList list;
	String os = null;
	DefaultMutableTreeNode ne = null;

	if (curnode.isNodeAncestor(lrctree))
	  list = rls.h.LRCGetPFNWC("*");
	else if ((rls.s.Flags & RLSStats.RLS_RCVLFNLIST) != 0)
	  list = rls.h.RLIGetLRCWC("*");
	else {
	  console.msg(rls.url + ": RLI updated by Bloom filters does not support wildcard search");
	  return;
	}
	Iterator iter = list.iterator();
	RLSString2 str2;
	while (iter.hasNext()) {
	  str2 = (RLSString2) iter.next();
	  if (!str2.s1.equals(os)) {
	    ne = new DefaultMutableTreeNode(str2.s1);
	    treemodel.insertNodeInto(ne, curnode, curnode.getChildCount());
	    os = str2.s1;
	  }
	  if (ne != null)
	    treemodel.insertNodeInto(new DefaultMutableTreeNode(str2.s2), ne, ne.getChildCount());
	}
      } catch (RLSException e) {
	console.msg(rls.url + ": " + e);
      }
    }

    void treeMousePressed(MouseEvent me) {
      TreePath tp = tree.getPathForLocation(me.getX(), me.getY());

      if (tp == null)
	return;
      DefaultMutableTreeNode n = (DefaultMutableTreeNode) tp.getLastPathComponent();
      Object o = n.getUserObject();

      if (n.isLeaf() && o instanceof String && n.isNodeAncestor(lrctree)) {
	if ((me.getModifiers() & MouseEvent.BUTTON1_MASK) != 0) {
	  setCursor(movecur);
	  dragging = true;
	  dragpfn = mkpfn(n);
	  o = ((DefaultMutableTreeNode) n.getParent()).getUserObject();
	  draglfn = o.toString();
	} else {
	  String pfn = mkpfn(n);
	  String s = "globus-url-copy " + pfn + " file://" + tf;
	  int r = RLSClient.OSSystem(s);
	  if (r != 0) {
	    console.msg(s + ": failed, status " + r);
	    return;
	  }
	  RLSClient.OSSystem("emacs " + tf);
	}
	return;
      }

      if (!(o instanceof RLSServer))
	return;
      curnode = n;
      if ((me.getModifiers() & MouseEvent.BUTTON1_MASK) != 0)
	selectedrls = (RLSServer) o;
      else {
	menurls = (RLSServer) o;
	popup.show(me.getComponent(), me.getX(), me.getY());
      }
    }

    void treeMouseReleased(MouseEvent me) {
      if (!dragging)
	return;
      dragging = false;
      setCursor(defcur);
      TreePath tp = tree.getPathForLocation(me.getX(), me.getY());

      if (tp == null)
	return;
      DefaultMutableTreeNode n = (DefaultMutableTreeNode) tp.getLastPathComponent();
      Object o = n.getUserObject();
      if (!(o instanceof RLSServer))
	return;
      if (!n.isNodeAncestor(lrctree))
	return;
      RLSServer rls = (RLSServer) o;
      try {
	URI suri = new URI(dragpfn);
	String destfile = "gsiftp://" + rls.uri.getHost() + suri.getPath();
	String cmd = "globus-url-copy " + dragpfn + " file://" + tf;
	int r = RLSClient.OSSystem(cmd);
	if (r == 0) {
	  cmd = "globus-url-copy file://" + tf + " " + destfile;
	  r = RLSClient.OSSystem(cmd);
	}
	if (r != 0) {
	  console.msg(cmd + ": failed, status " + r);
	  return;
	}
	try {
	  ((RLSServer) o).h.LRCCreate(draglfn, destfile);
	  console.msg("Created replica " + draglfn + ": " + destfile +
		      " successfully");
	} catch (RLSException ex) {
	  if (ex.GetRC() != RLSClient.RLS_LFN_EXIST) {
	    console.msg(ex.toString());
	    return;
	  }
	  String tpfn = destfile;
	  while (true) {
	    try {
	      ((RLSServer) o).h.LRCAdd(draglfn, tpfn);
	      console.msg("Created replica " + draglfn + ": " + tpfn +
			  " successfully");
	      break;
	    } catch (RLSException ex1) {
	      if (ex1.GetRC() == RLSClient.RLS_MAPPING_EXIST)
		tpfn = destfile + "." + tfnum++;
	      else {
		console.msg(ex1.toString());
		break;
	      }
	    }
	  }
	}
      } catch (Exception ex) {
	console.msg(ex.toString());
      }
    }
  }

  class ResultTree extends JPanel {
    DefaultMutableTreeNode root;
    DefaultTreeModel treemodel;
    JTree tree;
    DefaultMutableTreeNode curnode;
    String olds;
    DefaultMutableTreeNode oldn;
    boolean lrcres;

    public ResultTree() {
      root = new DefaultMutableTreeNode("Results");
      treemodel = new DefaultTreeModel(root);
      tree = new JTree(treemodel);
      tree.setEditable(true);
      tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
      tree.addMouseListener(new MouseAdapter() {
	public void mousePressed(MouseEvent me) {
	  treeMousePressed(me);
	}
      });
      olds = null;
      oldn = null;
      JScrollPane sp = new JScrollPane(tree);
      sp.setPreferredSize(new Dimension(RLW+RIW,TH-CH-BH));
      add(sp);
    }

    void clear(boolean llrcres) {
      root.removeAllChildren();
      treemodel.reload(root);
      lrcres = llrcres;
      olds = null;
      oldn = null;
    }

    void addnode(RLSString2 str2) {
      DefaultMutableTreeNode n;

      if (!str2.s1.equals(olds)) {
	n = new DefaultMutableTreeNode(str2.s1);
	treemodel.insertNodeInto(n, root, root.getChildCount());
	olds = str2.s1;
	oldn = n;
      } else
	n = oldn;
      treemodel.insertNodeInto(new DefaultMutableTreeNode(str2.s2), n, n.getChildCount());
    }

    void treeMousePressed(MouseEvent me) {
      TreePath tp = tree.getPathForLocation(me.getX(), me.getY());
      if (tp == null)
	return;
      DefaultMutableTreeNode n = (DefaultMutableTreeNode) tp.getLastPathComponent();
      if (!lrcres || n == root || !n.isLeaf())
	return;

      String s = "globus-url-copy " + mkpfn(n) + " file:///tmp/rlsgui.tmp";
      int r = RLSClient.OSSystem(s);
      if (r != 0) {
	console.msg(s + ": failed, status " + r);
	return;
      }
      RLSClient.OSSystem("emacs /tmp/rlsgui.tmp");
    }
  }

  class Console extends JPanel {
    public JTextArea ta;
    public JScrollPane sp;

    public Console() {
      ta = new JTextArea();
      sp = new JScrollPane(ta);
      sp.setPreferredSize(new Dimension(TW-20, CH));

      add(sp);
    }

    public void msg(String s) {
      ta.append(s + "\n");
    }
  }

  class RLSServer {
    public String url;
    public URI uri;
    public RLSClient h;
    public RLSStats s;

    public RLSServer(String lurl) {
      url = lurl;
      try {
	h = new RLSClient(url);
	s = h.Stats();
	uri = new URI(url);
      } catch (Exception e) {
	System.out.println(url + ": " + e);
	h = null;
      }
    }

    public String toString() {
      return url;
    }

    public void UpdateStats() {
      try {
	s = h.Stats();
      } catch (RLSException e) {
	console.msg(url + ": " + e);
      }
    }
  }

  public void actionPerformed(ActionEvent e) {
    JMenuItem source = (JMenuItem) (e.getSource());
    String s = source.getText();
    Topology top;
    JFrame topframe;
    Container topcontents;
    Iterator iter;
    ArrayList list;
    RLSRLIInfo ri;
    RLSLRCInfo li;
    JScrollPane sp;

    if (s.equals("Get LFNs"))
      rlstree.GetLFNs(menurls);
    else if (s.equals("Info")) {
      menurls.UpdateStats();
      int ut = menurls.s.Uptime;
      int d = ut / (24 * 3600);
      ut = ut % (24 * 3600);
      String msg = "URL: " + menurls.url +
		   "\nUptime: " + d + "d " +
				  ut / 3600 + ":" +
				  (ut / 60) % 60 + ":" +
				  ut % 60;
      if ((menurls.s.Flags & RLSStats.RLS_SNDLFNLIST) != 0)
	msg += "\nUpdate method: Sends LFN Lists";
      if ((menurls.s.Flags & RLSStats.RLS_SNDBLOOMFILTER) != 0)
	msg += "\nUpdate method: Sends Bloom Filters";
      if ((menurls.s.Flags & RLSStats.RLS_RCVLFNLIST) != 0)
	msg += "\nUpdate method: Receives LFN Lists";
      if ((menurls.s.Flags & RLSStats.RLS_RCVBLOOMFILTER) != 0)
	msg += "\nUpdate method: Receives Bloom Filters";
      if ((menurls.s.Flags & RLSStats.RLS_LRCSERVER) != 0) {
	try {
	  list = menurls.h.LRCRLIList();
	  iter = list.iterator();
	  while (iter.hasNext()) {
	    ri = (RLSRLIInfo) iter.next();
	    if ((ri.flags & RLSClient.RLIFLAG_BLOOMFILTER) != 0)
	      msg += "\nUpdatesBloomFilter: " + ri.url + 
		     " Last update: " + ri.lastupdate;
	    else
	      msg += "\nUpdatesLFNList: " + ri.url +
		     " Last update: " + ri.lastupdate;
	  }
	} catch (RLSException ex) {
	  if (ex.GetRC() != RLSClient.RLS_RLI_NEXIST)
	    console.msg(ex.toString());
	}
	msg += "\nLRC LFN Count: " + menurls.s.LRCNumLFN;
	msg += "\nLRC PFN Count: " + menurls.s.LRCNumPFN;
	msg += "\nLRC MAP Count: " + menurls.s.LRCNumMAP;
      }
      if ((menurls.s.Flags & RLSStats.RLS_RLISERVER) != 0) {
	try {
	  list = menurls.h.RLILRCList();
	  iter = list.iterator();
	  while (iter.hasNext()) {
	    li = (RLSLRCInfo) iter.next();
	    msg += "\nUpdatedBy: " + li.url + " Last update: " + li.lastupdate;
	  }
	} catch (RLSException ex) {
	  if (ex.GetRC() != RLSClient.RLS_LRC_NEXIST)	    
	    console.msg(ex.toString());
	}
	if ((menurls.s.Flags & RLSStats.RLS_RCVLFNLIST) != 0) {
	  msg += "\nRLI LFN Count: " + menurls.s.RLINumLFN;
	  msg += "\nRLI LRC Count: " + menurls.s.RLINumLRC;
	  msg += "\nRLI Sender Count: " + menurls.s.RLINumSender;
	  msg += "\nRLI MAP Count: " + menurls.s.RLINumMAP;
	}
      }
      JOptionPane.showMessageDialog(frame, msg);
    } else if (s.equals("Topology")) {
      try {
        top = new Topology(menurls);
      } catch (RLSException ex) {
	console.msg(ex.toString());
	return;
      }
      topframe = new JFrame(menurls + " Topology");
      topcontents = topframe.getContentPane();
      sp = new JScrollPane(top);
      sp.setPreferredSize(new Dimension(1000,TOPH));
      topcontents.add(sp);
      topframe.pack();
      topframe.setVisible(true);
    } else if (s.equals("View Topology")) {
      try {
        top = new Topology();
      } catch (RLSException ex) {
	console.msg(ex.toString());
	return;
      }
      topframe = new JFrame("RLS Server Topology");
      topcontents = topframe.getContentPane();
      sp = new JScrollPane(top);
      sp.setPreferredSize(new Dimension(1000,TOPH));
      topcontents.add(sp);
      topframe.pack();
      topframe.setVisible(true);
    } else if (s.equals("Update")) {
      rlstree.Update(menurls);
    } else if (s.equals("Quit")) {
      rlstree.CloseAll();
      System.exit(0);
    }
  }

  public class Topology extends JPanel {
    ArrayList lrcs;
    ArrayList rlis;
    boolean conn[][];

    public Topology() throws RLSException {
      Iterator iter;
      RLSServer rlsserver;
      ArrayList rliurls;
      RLSRLIInfo ri;
      int i;
      int j;
      int ridx;

      conn = new boolean[rlstree.lrcs.size()][200];
      lrcs = new ArrayList(rlstree.lrcs.size());
      rlis = new ArrayList();
      rliurls = new ArrayList();
      i = 0;
      ridx = 0;
      iter = rlstree.lrcs.iterator();
      while (iter.hasNext()) {
	// Create lrcs list from LRC servers on rlstree.lrcs
	rlsserver = (RLSServer) iter.next();
	RLSLRCInfo li = new RLSLRCInfo(rlsserver.url, 0);
	lrcs.add(li);

	// Get RLIs updated by this LRC and add to rlis list
	try {
	  ArrayList p = rlsserver.h.LRCRLIList();
	  Iterator pi = p.iterator();
	  while (pi.hasNext()) {
	    ri = (RLSRLIInfo) pi.next();
	    if (rliurls.contains(ri.url.toLowerCase())) {
	      j = rliurls.indexOf(ri.url.toLowerCase());
	      conn[i][j] = true;
	    } else {
	      rliurls.add(ridx, ri.url.toLowerCase());
	      rlis.add(ridx, ri);
	      conn[i][ridx] = true;
	      ridx++;
	    }
	  }
	} catch (RLSException ex) {
	  if (ex.GetRC() != RLSClient.RLS_RLI_NEXIST)
	    console.msg(ex.toString());
        }
	i++;
      }
      int w;
      if ((w = lrcs.size()) < rlis.size())
	w = rlis.size();
      w = w * 155 + 5;
      setPreferredSize(new Dimension(w, 200));
    }

    public Topology(RLSServer rls) throws RLSException {
      int w;
      int i;

      if (rlstree.curnode.isNodeAncestor(rlstree.lrctree)) {
	rlis = rls.h.LRCRLIList();
	RLSLRCInfo li = new RLSLRCInfo(rls.url, 0);
	lrcs = new ArrayList(1);
	lrcs.add(li);
	w = rlis.size() * 155 + 5;
	conn = new boolean[1][rlis.size()];
	for (i = 0; i < rlis.size(); i++)
	  conn[0][i] = true;
      } else {
	RLSRLIInfo ri = new RLSRLIInfo(rls.url, 0, (rls.s.Flags & RLSStats.RLS_RCVBLOOMFILTER) == 0 ? 0 : RLSClient.RLIFLAG_BLOOMFILTER, 0);
	lrcs = rls.h.RLILRCList();
	rlis = new ArrayList(1);
	rlis.add(ri);
	w = lrcs.size() * 155 + 5;
	conn = new boolean[lrcs.size()][1];
	for (i = 0; i < lrcs.size(); i++)
	  conn[i][0] = true;
      }
      setPreferredSize(new Dimension(w, 200));
    }

    public void paint(Graphics g) {
      Graphics2D g2 = (Graphics2D) g;
      Dimension d = getSize();
      Iterator iter;
      RLSRLIInfo ri;
      RLSLRCInfo li;
      int i;
      int j;
      int w;
      URI uri;
      int radj;
      int ladj;

      g2.setBackground(Color.black);
      g2.clearRect(0, 0, d.width, d.height);
      g2.setColor(Color.cyan);
      g2.drawString("Bloom Filter", 5, 20);
      g2.setColor(Color.green);
      g2.drawString("LFN List", 85, 20);
      g2.setColor(Color.lightGray);
      g2.drawString("Both", 165, 20);

      w = rlis.size() * 155 + 5;
      if (w < d.width)
	radj = (d.width - w) / 2 + 5;
      else
	radj = 5;
      iter = rlis.iterator();
      i = 0;
      while (iter.hasNext()) {
	ri = (RLSRLIInfo) iter.next();
	if ((ri.flags & RLSClient.RLIFLAG_BLOOMFILTER) == 0)
	  g2.setColor(Color.green);
	else
	  g2.setColor(Color.cyan);
	g2.fill3DRect(i * 155 + radj, 45, 150, 40, true);
	try {
	  uri = new URI(ri.url);
	  g2.setColor(Color.black);
	  g2.drawString(uri.getHost(), i * 155 + radj + 5, 45 + 20);
	} catch (Exception ex) {
	  console.msg(ex.toString());
	}
	i++;
      }

      w = lrcs.size() * 155 + 5;
      if (w < d.width)
	ladj = (d.width - w) / 2 + 5;
      else
	ladj = 5;
      iter = lrcs.iterator();
      i = 0;
      while (iter.hasNext()) {
	li = (RLSLRCInfo) iter.next();
	g2.setColor(Color.lightGray);
	g2.fill3DRect(i * 155 + ladj, TOPH-90, 150, 40, true);
	try {
	  uri = new URI(li.url);
	  g2.setColor(Color.black);
	  g2.drawString(uri.getHost(), i * 155 + ladj + 5, TOPH-70);
	} catch (Exception ex) {
	  console.msg(ex.toString());
	}
	i++;
      }

      g2.setColor(Color.white);
      for (i = 0; i < lrcs.size(); i++)
	for (j = 0; j < rlis.size(); j++) {
	  if (!conn[i][j])
	    continue;
	  g2.drawLine(i * 155 + ladj + 77, TOPH-90, j * 155 + radj + 77, 85);
	}
    }
  }

  public static void main(String[] args) {
    if (args.length == 0) {
      System.out.println("Usage: RLSgui rls-server1 ...");
      System.exit(0);
    }
    RLSGui window = new RLSGui(args);
  }

  public void itemStateChanged(ItemEvent e) {
    System.out.println("item state changed");
  }

  public String mkpfn(RLSServer rls, String pfn) {
    if (pfn.indexOf(':') == -1)
      if (pfn.charAt(0) == '/')
	return "gsiftp://" + rls.uri.getHost() + pfn;
      else
	return "gsiftp://" + pfn;
    else
      return pfn;
  }

  public String mkpfn(DefaultMutableTreeNode n) {
    String pfn = n.getUserObject().toString();
    if (pfn.indexOf(':') > 0)
      return pfn;
    if (pfn.charAt(0) != '/')
      return "gsiftp://" + pfn;
    n = (DefaultMutableTreeNode) n.getParent();
    n = (DefaultMutableTreeNode) n.getParent();
    RLSServer rls = (RLSServer) n.getUserObject();
    return "gsiftp://" + rls.uri.getHost() + pfn;
  }

}
