/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <globus_rls_client.h>
#include "RLSClient.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <jni.h>

static globus_module_descriptor_t	*modules[] = {
  GLOBUS_COMMON_MODULE,
  GLOBUS_IO_MODULE,
  GLOBUS_RLS_CLIENT_MODULE,
};
#define NMODS	(sizeof(modules) / sizeof(globus_module_descriptor_t *))

static jobject	makeattrlist(JNIEnv * env, globus_list_t *list, int v);
static jobject	makeattrobjlist(JNIEnv * env, globus_list_t *list);
static jobject	makestringlist(JNIEnv *env, globus_list_t *list);
static jobject	makestring2list(JNIEnv *env, globus_list_t *list);
static jobject	makestring2bulklist(JNIEnv *env, globus_list_t *list);
static void	freestrlist(globus_list_t *strlist);
static void	freestr2list(globus_list_t *str2list);
static void	freeattrobjlist(globus_list_t *attrobjlist);
static void	jofflim2offlim(JNIEnv *env, jobject jofflim, jfieldID *ofid,
			       int *offset, int *reslimit);
static jobject	attr2jattr(JNIEnv *env, globus_rls_attribute_t *a, int v,
			   jclass ac);
static void	jattr2attr(JNIEnv *env, jobject jattr,
			   globus_rls_attribute_t *a);
static void	throwexception(JNIEnv *env, globus_result_t r);
static void	throwexception1(JNIEnv *env, int rc, char *errmsg);

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_init(JNIEnv *env, jobject obj)

{
  int	i;

  for (i = 0; i < NMODS; i++)
    globus_module_activate(modules[i]);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_admin(JNIEnv *env, jobject obj,
					    jint handle, jint cmd)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_admin(h, (globus_rls_admin_cmd_t) cmd);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_certificate(JNIEnv *env, jobject obj,
						  jstring jcert, jstring jkey)

{
  char	*cert;
  char	*key;

  cert = (char *) (*env)->GetStringUTFChars(env, jcert, NULL);
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  globus_rls_client_certificate(cert, key);
  (*env)->ReleaseStringUTFChars(env, jcert, cert);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_proxycertificate(JNIEnv *env,
	jobject obj, jstring jproxy)
						  

{
  char	*proxy;

  proxy = (char *) (*env)->GetStringUTFChars(env, jproxy, NULL);
  globus_rls_client_proxy_certificate(proxy);
  (*env)->ReleaseStringUTFChars(env, jproxy, proxy);
}

JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_connect(JNIEnv *env, jobject obj, jstring jurl)

{
  char			*url;
  globus_result_t	r;
  globus_rls_handle_t	*h;

  url = (char *) (*env)->GetStringUTFChars(env, jurl, NULL);
  r = globus_rls_client_connect(url, &h);
  (*env)->ReleaseStringUTFChars(env, jurl, url);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return 0;
  }
  return (int) h;
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_close(JNIEnv *env, jobject obj,jint handle)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_close(h);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jstring JNICALL
Java_org_globus_replica_rls_RLSClient_geterrormessage(JNIEnv *env, jobject obj,
						      jint rc)

{
  char	errmsg[MAXERRMSG];

  globus_rls_errmsg(rc, NULL, errmsg, MAXERRMSG);
  return (*env)->NewStringUTF(env, errmsg);
}

JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_gettimeout(JNIEnv *env, jobject obj)

{
  return globus_rls_client_get_timeout();
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_settimeout(JNIEnv *env, jobject obj,
						 jint seconds)

{
  globus_rls_client_set_timeout(seconds);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattradd(JNIEnv *env, jobject obj,
			jint handle, jstring jkey, jobject jattr)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  char				*key;
  globus_rls_attribute_t	a;

  h = (globus_rls_handle_t *) handle;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  jattr2attr(env, jattr, &a);
  r = globus_rls_client_lrc_attr_add(h, key, &a);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattraddbulk(JNIEnv *env, jobject obj,
						jint handle, jint attrobjlist)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  globus_list_t			*al;
  globus_list_t			*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) attrobjlist;
  r = globus_rls_client_lrc_attr_add_bulk(h, al, &rl);
  freeattrobjlist(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrremovebulk(JNIEnv *env,
				jobject obj, jint handle, jint attrobjlist)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  globus_list_t			*al;
  globus_list_t			*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) attrobjlist;
  r = globus_rls_client_lrc_attr_remove_bulk(h, al, &rl);
  freeattrobjlist(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrcreate(JNIEnv *env, jobject obj,
			jint handle, jstring jname, jint objtype, jint valtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*name;

  h = (globus_rls_handle_t *) handle;
  name = (char *) (*env)->GetStringUTFChars(env, jname, NULL);
  r = globus_rls_client_lrc_attr_create(h, name, objtype, valtype);
  (*env)->ReleaseStringUTFChars(env, jname, name);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrdelete(JNIEnv *env, jobject obj,
		jint handle, jstring jname, jint objtype, jint clearvalues)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*name;

  h = (globus_rls_handle_t *) handle;
  name = (char *) (*env)->GetStringUTFChars(env, jname, NULL);
  r = globus_rls_client_lrc_attr_delete(h, name, objtype, clearvalues);
  (*env)->ReleaseStringUTFChars(env, jname, name);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrget(JNIEnv *env, jobject obj,
				jint handle, jstring jname, jint objtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*name;
  globus_list_t		*list;

  h = (globus_rls_handle_t *) handle;
  if (jname)
    name = (char *) (*env)->GetStringUTFChars(env, jname, NULL);
  else
    name = NULL;
  r = globus_rls_client_lrc_attr_get(h, name, objtype, &list);
  if (name)
    (*env)->ReleaseStringUTFChars(env, jname, name);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makeattrlist(env, list, 0);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrmodify(JNIEnv *env, jobject obj,
				jint handle, jstring jkey, jobject jattr)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  char				*key;
  globus_rls_attribute_t	a;

  h = (globus_rls_handle_t *) handle;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  jattr2attr(env, jattr, &a);
  r = globus_rls_client_lrc_attr_modify(h, key, &a);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrremove(JNIEnv *env, jobject obj,
				jint handle, jstring jkey, jobject jattr)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  char				*key;
  globus_rls_attribute_t	a;

  h = (globus_rls_handle_t *) handle;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  jattr2attr(env, jattr, &a);
  r = globus_rls_client_lrc_attr_remove(h, key, &a);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrsearch(JNIEnv *env, jobject obj,
		jint handle, jstring jname, jint objtype, jint op, jobject jop1,
		jobject jop2, jobject jofflim)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  char				*name;
  globus_rls_attribute_t	op1;
  globus_rls_attribute_t	op2;
  globus_rls_attribute_t	*op1p;
  globus_rls_attribute_t	*op2p;
  globus_list_t			*list;
  jfieldID			ofid;
  int				offset;
  int				reslimit;

  h = (globus_rls_handle_t *) handle;
  if (jop1) {
    jattr2attr(env, jop1, &op1);
    op1p = &op1;
  } else
    op1p = NULL;
  if (jop2) {
    jattr2attr(env, jop2, &op2);
    op2p = &op2;
  } else
    op2p = NULL;
  name = (char *) (*env)->GetStringUTFChars(env, jname, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_lrc_attr_search(h, name, objtype, op, op1p, op2p,
					  NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_lrc_attr_search(h, name, objtype, op, op1p, op2p,
					  &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jname, name);
  if (op1p) {
    free(op1p->name);
    if (op1p->type == globus_rls_attr_type_str)
      free(op1p->val.s);
  }
  if (op2p) {
    free(op2p->name);
    if (op2p->type == globus_rls_attr_type_str)
      free(op2p->val.s);
  }
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makeattrobjlist(env, list);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrvalueget(JNIEnv *env, jobject obj,
			jint handle, jstring jkey, jstring jname, jint objtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*key;
  char			*name;
  globus_list_t		*list;

  h = (globus_rls_handle_t *) handle;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  if (jname)
    name = (char *) (*env)->GetStringUTFChars(env, jname, NULL);
  else
    name = NULL;
  r = globus_rls_client_lrc_attr_value_get(h, key, name, objtype, &list);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  if (name)
    (*env)->ReleaseStringUTFChars(env, jname, name);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makeattrlist(env, list, 1);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcattrvaluegetbulk(JNIEnv *env,
	jobject obj, jint handle, jint keylist, jstring jname, jint objtype)

{
  globus_result_t		r;
  globus_rls_handle_t		*h;
  char				*name;
  globus_list_t			*al;
  globus_list_t			*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) keylist;
  if (jname)
    name = (char *) (*env)->GetStringUTFChars(env, jname, NULL);
  else
    name = NULL;
  r = globus_rls_client_lrc_attr_value_get_bulk(h, al, name, objtype, &rl);
  freestrlist(al);
  if (name)
    (*env)->ReleaseStringUTFChars(env, jname, name);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makeattrobjlist(env, rl);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcadd(JNIEnv *env, jobject obj,
			       jint handle, jstring jlfn, jstring jpfn)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  char			*pfn;

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  pfn = (char *) (*env)->GetStringUTFChars(env, jpfn, NULL);
  r = globus_rls_client_lrc_add(h, lfn, pfn);
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  (*env)->ReleaseStringUTFChars(env, jpfn, pfn);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcaddbulk(JNIEnv *env, jobject obj,
						 jint handle, jint str2list)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*al;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) str2list;
  r = globus_rls_client_lrc_add_bulk(h, al, &rl);
  freestr2list(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcclear(JNIEnv *env, jobject obj, jint handle)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_lrc_clear(h);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrccreate(JNIEnv *env, jobject obj,
			       jint handle, jstring jlfn, jstring jpfn)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  char			*pfn;

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  pfn = (char *) (*env)->GetStringUTFChars(env, jpfn, NULL);
  r = globus_rls_client_lrc_create(h, lfn, pfn);
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  (*env)->ReleaseStringUTFChars(env, jpfn, pfn);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrccreatebulk(JNIEnv *env, jobject obj,
						    jint handle, jint str2list)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*al;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) str2list;
  r = globus_rls_client_lrc_create_bulk(h, al, &rl);
  freestr2list(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcdelete(JNIEnv *env, jobject obj,
			       jint handle, jstring jlfn, jstring jpfn)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  char			*pfn;

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  pfn = (char *) (*env)->GetStringUTFChars(env, jpfn, NULL);
  r = globus_rls_client_lrc_delete(h, lfn, pfn);
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  (*env)->ReleaseStringUTFChars(env, jpfn, pfn);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcdeletebulk(JNIEnv *env, jobject obj,
						    jint handle, jint str2list)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*al;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) str2list;
  r = globus_rls_client_lrc_delete_bulk(h, al, &rl);
  freestr2list(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT jboolean JNICALL
Java_org_globus_replica_rls_RLSClient_lrcexists(JNIEnv *env, jobject obj,
			       jint handle, jstring jkey, jint objtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*key;
  int			rc;
  char			errmsg[MAXERRMSG];

  h = (globus_rls_handle_t *) handle;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  r = globus_rls_client_lrc_exists(h, key, objtype);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  if (r == GLOBUS_SUCCESS)
    return (jboolean) 1;
  globus_rls_client_error_info(r, &rc, errmsg, MAXERRMSG, GLOBUS_FALSE);
  if (rc != GLOBUS_RLS_LFN_NEXIST && rc != GLOBUS_RLS_PFN_NEXIST)
    throwexception1(env, rc, errmsg);
  return (jboolean) 0;
}

JNIEXPORT jobject JNICALL Java_org_globus_replica_rls_RLSClient_lrcexistsbulk
  (JNIEnv *env, jobject obj, jint handle, jint strlist, jint objtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*keylist;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  keylist = (globus_list_t *) strlist;
  r = globus_rls_client_lrc_exists_bulk(h, keylist, objtype, &rl);
  freestrlist(keylist);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcgetlfn(JNIEnv *env, jobject obj,
			jint handle, jstring jpfn, jobject jofflim)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*pfn;
  globus_list_t		*list;
  int			offset;
  int			reslimit;
  jfieldID		ofid;

  h = (globus_rls_handle_t *) handle;
  pfn = (char *) (*env)->GetStringUTFChars(env, jpfn, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_lrc_get_lfn(h, pfn, NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_lrc_get_lfn(h, pfn, &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jpfn, pfn);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcgetlfnbulk(JNIEnv *env, jobject obj,
			jint handle, jint strlist)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*al;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) strlist;
  r = globus_rls_client_lrc_get_lfn_bulk(h, al, &rl);
  freestrlist(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2bulklist(env, rl);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcgetlfnwc(JNIEnv *env, jobject obj,
				jint handle, jstring jpfnpat, jobject jofflim)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*pfnpat;
  globus_list_t		*list;
  int			offset;
  int			reslimit;
  jfieldID		ofid;

  h = (globus_rls_handle_t *) handle;
  pfnpat = (char *) (*env)->GetStringUTFChars(env, jpfnpat, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_lrc_get_lfn_wc(h, pfnpat, rls_pattern_unix,
					 NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_lrc_get_lfn_wc(h, pfnpat, rls_pattern_unix,
					 &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jpfnpat, pfnpat);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcgetpfn(JNIEnv *env, jobject obj,
				jint handle, jstring jlfn, jobject jofflim)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  globus_list_t		*list;
  int			offset;
  int			reslimit;
  jfieldID		ofid;

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_lrc_get_pfn(h, lfn, NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_lrc_get_pfn(h, lfn, &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcgetpfnbulk(JNIEnv *env, jobject obj,
			jint handle, jint strlist)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*al;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) strlist;
  r = globus_rls_client_lrc_get_pfn_bulk(h, al, &rl);
  freestrlist(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2bulklist(env, rl);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcgetpfnwc(JNIEnv *env, jobject obj,
				jint handle, jstring jlfnpat, jobject jofflim)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfnpat;
  globus_list_t		*list;
  int			offset;
  int			reslimit;
  jfieldID		ofid;

  h = (globus_rls_handle_t *) handle;
  lfnpat = (char *) (*env)->GetStringUTFChars(env, jlfnpat, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_lrc_get_pfn_wc(h, lfnpat, rls_pattern_unix,
					 NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_lrc_get_pfn_wc(h, lfnpat, rls_pattern_unix,
					 &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jlfnpat, lfnpat);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jboolean JNICALL
Java_org_globus_replica_rls_RLSClient_lrcmappingexists(JNIEnv *env,jobject obj,
			       jint handle, jstring jlfn, jstring jpfn)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  char			*pfn;
  int			rc;
  char			errmsg[MAXERRMSG];

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  pfn = (char *) (*env)->GetStringUTFChars(env, jpfn, NULL);
  r = globus_rls_client_lrc_mapping_exists(h, lfn, pfn);
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  (*env)->ReleaseStringUTFChars(env, jpfn, pfn);
  if (r == GLOBUS_SUCCESS)
    return (jboolean) 1;
  globus_rls_client_error_info(r, &rc, errmsg, MAXERRMSG, GLOBUS_FALSE);
  if (rc != GLOBUS_RLS_MAPPING_NEXIST)
    throwexception1(env, rc, errmsg);
  return (jboolean) 0;
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrenamelfn(JNIEnv *env, jobject obj,
			       jint handle, jstring joldname, jstring jnewname)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*oldname;
  char			*newname;

  h = (globus_rls_handle_t *) handle;
  oldname = (char *) (*env)->GetStringUTFChars(env, joldname, NULL);
  newname = (char *) (*env)->GetStringUTFChars(env, jnewname, NULL);
  r = globus_rls_client_lrc_renamelfn(h, oldname, newname);
  (*env)->ReleaseStringUTFChars(env, joldname, oldname);
  (*env)->ReleaseStringUTFChars(env, jnewname, newname);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrenamelfnbulk(JNIEnv *env, jobject obj,
						 jint handle, jint str2list)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*renamelist;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  renamelist = (globus_list_t *) str2list;
  r = globus_rls_client_lrc_renamelfn_bulk(h, renamelist, &rl);
  freestr2list(renamelist);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrenamepfn(JNIEnv *env, jobject obj,
			       jint handle, jstring joldname, jstring jnewname)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*oldname;
  char			*newname;

  h = (globus_rls_handle_t *) handle;
  oldname = (char *) (*env)->GetStringUTFChars(env, joldname, NULL);
  newname = (char *) (*env)->GetStringUTFChars(env, jnewname, NULL);
  r = globus_rls_client_lrc_renamepfn(h, oldname, newname);
  (*env)->ReleaseStringUTFChars(env, joldname, oldname);
  (*env)->ReleaseStringUTFChars(env, jnewname, newname);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrenamepfnbulk(JNIEnv *env, jobject obj,
						 jint handle, jint str2list)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*renamelist;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  renamelist = (globus_list_t *) str2list;
  r = globus_rls_client_lrc_renamepfn_bulk(h, renamelist, &rl);
  freestr2list(renamelist);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrliadd(JNIEnv *env, jobject obj, jint handle,
			     jstring jrliurl, jint flags, jstring jpattern)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*rliurl;
  char			*pattern;

  h = (globus_rls_handle_t *) handle;
  rliurl = (char *) (*env)->GetStringUTFChars(env, jrliurl, NULL);
  if (jpattern)
    pattern = (char *) (*env)->GetStringUTFChars(env, jpattern, NULL);
  else
    pattern = NULL;
  r = globus_rls_client_lrc_rli_add(h, rliurl, flags, pattern);
  (*env)->ReleaseStringUTFChars(env, jrliurl, rliurl);
  if (jpattern)
    (*env)->ReleaseStringUTFChars(env, jpattern, pattern);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT void JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrlidelete(JNIEnv *env, jobject obj, jint handle,
			     jstring jrliurl, jstring jpattern)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*rliurl;
  char			*pattern;

  h = (globus_rls_handle_t *) handle;
  rliurl = (char *) (*env)->GetStringUTFChars(env, jrliurl, NULL);
  if (jpattern)
    pattern = (char *) (*env)->GetStringUTFChars(env, jpattern, NULL);
  else
    pattern = NULL;
  r = globus_rls_client_lrc_rli_delete(h, rliurl, pattern);
  (*env)->ReleaseStringUTFChars(env, jrliurl, rliurl);
  if (jpattern)
    (*env)->ReleaseStringUTFChars(env, jpattern, pattern);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrligetpart(JNIEnv *env, jobject obj, jint handle,
			        jstring jrliurl, jstring jpattern)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*rliurl;
  char			*pattern;
  globus_list_t		*list;

  h = (globus_rls_handle_t *) handle;
  if (jrliurl)
    rliurl = (char *) (*env)->GetStringUTFChars(env, jrliurl, NULL);
  else
    rliurl = NULL;
  if (jpattern)
    pattern = (char *) (*env)->GetStringUTFChars(env, jpattern, NULL);
  else
    pattern = NULL;
  r = globus_rls_client_lrc_rli_get_part(h, rliurl, pattern, &list);
  if (jrliurl)
    (*env)->ReleaseStringUTFChars(env, jrliurl, rliurl);
  if(jpattern)
    (*env)->ReleaseStringUTFChars(env, jpattern, pattern);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2list(env, list);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrliinfo(JNIEnv *env, jobject obj, jint handle,
			      jstring jrliurl)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*rliurl;
  globus_rls_rli_info_t	info;
  jclass		ic;
  jmethodID		mid;

  h = (globus_rls_handle_t *) handle;
  rliurl = (char *) (*env)->GetStringUTFChars(env, jrliurl, NULL);
  r = globus_rls_client_lrc_rli_info(h, rliurl, &info);
  (*env)->ReleaseStringUTFChars(env, jrliurl, rliurl);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  ic = (*env)->FindClass(env, "org/globus/replica/rls/RLSRLIInfo");
  mid = (*env)->GetMethodID(env, ic, "<init>", "(Ljava/lang/String;III)V");
  return (*env)->NewObject(env, ic, mid, (*env)->NewStringUTF(env, info.url),
			   info.updateinterval, info.flags, info.lastupdate);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_lrcrlilist(JNIEnv *env, jobject obj, jint handle)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*list;
  globus_list_t		*p;
  jclass		lc;
  jmethodID		lcid;
  jmethodID		laid;
  jobject		lo;
  jclass		ic;
  jmethodID		mid;
  jobject		io;
  int			len;
  globus_rls_rli_info_t	*rliinfo;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_lrc_rli_list(h, &list);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  /* Create ArrayList object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get ids necessary to create RLIInfo objects	*/
  ic = (*env)->FindClass(env, "org/globus/replica/rls/RLSRLIInfo");
  mid = (*env)->GetMethodID(env, ic, "<init>", "(Ljava/lang/String;III)V");

  for (p = list; p; p = globus_list_rest(p)) {
    rliinfo = (globus_rls_rli_info_t *) globus_list_first(p);
    io = (*env)->NewObject(env, ic, mid,
			   (*env)->NewStringUTF(env, rliinfo->url),
			   rliinfo->updateinterval, rliinfo->flags,
			   rliinfo->lastupdate);
    (*env)->CallBooleanMethod(env, lo, laid, io);
  }
  globus_rls_client_free_list(list);
  return lo;
}

JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_makestrlist(JNIEnv *env, jobject obj,
						  jint str2list, jstring jlfn)

{
  globus_list_t		*p;
  char			*lfn;

  p = (globus_list_t *) str2list;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  globus_list_insert(&p, strdup(lfn));
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  return (int) p;
}

JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_makestr2list(JNIEnv *env, jobject obj,
				jint str2list, jstring jlfn, jstring jpfn)

{
  globus_list_t		*p;
  char			*lfn;
  char			*pfn;
  globus_rls_string2_t	*str2;

  p = (globus_list_t *) str2list;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  pfn = (char *) (*env)->GetStringUTFChars(env, jpfn, NULL);
  str2 = (globus_rls_string2_t *) malloc(sizeof(globus_rls_string2_t));
  str2->s1 = strdup(lfn);
  str2->s2 = strdup(pfn);
  globus_list_insert(&p, str2);
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  (*env)->ReleaseStringUTFChars(env, jpfn, pfn);
  return (int) p;
}

JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_makeaolist(JNIEnv *env, jobject obj,
				jint attrobjlist, jstring jkey, jobject jattr)

{
  globus_list_t			*p;
  char				*key;
  globus_rls_attribute_object_t	*ao;

  p = (globus_list_t *) attrobjlist;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  ao = (globus_rls_attribute_object_t *) malloc(sizeof(globus_rls_attribute_object_t));
  ao->key = strdup(key);
  jattr2attr(env, jattr, &ao->attr);
  globus_list_insert(&p, ao);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  return (int) p;
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_rligetlrc(JNIEnv *env, jobject obj,
				jint handle, jstring jlfn, jobject jofflim)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  globus_list_t		*list;
  int			offset;
  int			reslimit;
  jfieldID		ofid;

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_rli_get_lrc(h, lfn, NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_rli_get_lrc(h, lfn, &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_rligetlrcbulk(JNIEnv *env, jobject obj,
			jint handle, jint strlist)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*al;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  al = (globus_list_t *) strlist;
  r = globus_rls_client_rli_get_lrc_bulk(h, al, &rl);
  freestrlist(al);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2bulklist(env, rl);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_rligetlrcwc(JNIEnv *env, jobject obj,
				jint handle, jstring jlfnpat, jobject jofflim)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfnpat;
  globus_list_t		*list;
  int			offset;
  int			reslimit;
  jfieldID		ofid;

  h = (globus_rls_handle_t *) handle;
  lfnpat = (char *) (*env)->GetStringUTFChars(env, jlfnpat, NULL);
  if (jofflim == NULL)
    r = globus_rls_client_rli_get_lrc_wc(h, lfnpat, rls_pattern_unix,
					 NULL, 0, &list);
  else {
    jofflim2offlim(env, jofflim, &ofid, &offset, &reslimit);
    r = globus_rls_client_rli_get_lrc_wc(h, lfnpat, rls_pattern_unix,
					 &offset, reslimit, &list);
    if (r == GLOBUS_SUCCESS)
      (*env)->SetIntField(env, jofflim, ofid, (jint) offset);
  }
  (*env)->ReleaseStringUTFChars(env, jlfnpat, lfnpat);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jboolean JNICALL
Java_org_globus_replica_rls_RLSClient_rliexists(JNIEnv *env, jobject obj,
			       jint handle, jstring jkey, jint objtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*key;
  int			rc;
  char			errmsg[MAXERRMSG];

  h = (globus_rls_handle_t *) handle;
  key = (char *) (*env)->GetStringUTFChars(env, jkey, NULL);
  r = globus_rls_client_rli_exists(h, key, objtype);
  (*env)->ReleaseStringUTFChars(env, jkey, key);
  if (r == GLOBUS_SUCCESS)
    return (jboolean) 1;
  globus_rls_client_error_info(r, &rc, errmsg, MAXERRMSG, GLOBUS_FALSE);
  if (rc != GLOBUS_RLS_LFN_NEXIST && rc != GLOBUS_RLS_LRC_NEXIST)
    throwexception1(env, rc, errmsg);
  return (jboolean) 0;
}

JNIEXPORT jobject JNICALL Java_org_globus_replica_rls_RLSClient_rliexistsbulk
  (JNIEnv *env, jobject obj, jint handle, jint strlist, jint objtype)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*keylist;
  globus_list_t		*rl;

  h = (globus_rls_handle_t *) handle;
  keylist = (globus_list_t *) strlist;
  r = globus_rls_client_rli_exists_bulk(h, keylist, objtype, &rl);
  freestrlist(keylist);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }
  return makestring2bulklist(env, rl);
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_rlisenderlist(JNIEnv *env, jobject obj,
						    jint handle)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*list;
  globus_list_t		*p;
  jclass		lc;
  jmethodID		lcid;
  jmethodID		laid;
  jobject		lo;
  jclass		ic;
  jmethodID		mid;
  jobject		io;
  int			len;
  globus_rls_lrc_info_t	*lrcinfo;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_rli_sender_list(h, &list);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  /* Create List object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get ids necessary to create LRCInfo objects	*/
  ic = (*env)->FindClass(env, "org/globus/replica/rls/RLSLRCInfo");
  mid = (*env)->GetMethodID(env, ic, "<init>", "(Ljava/lang/String;I)V");

  for (p = list; p; p = globus_list_rest(p)) {
    lrcinfo = (globus_rls_lrc_info_t *) globus_list_first(p);
    io = (*env)->NewObject(env, ic, mid,
			   (*env)->NewStringUTF(env, lrcinfo->url),
			   lrcinfo->lastupdate);
    (*env)->CallBooleanMethod(env, lo, laid, io);
  }
  globus_rls_client_free_list(list);
  return lo;
}

JNIEXPORT jboolean JNICALL
Java_org_globus_replica_rls_RLSClient_rlimappingexists(JNIEnv *env,jobject obj,
			       jint handle, jstring jlfn, jstring jlrc)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*lfn;
  char			*lrc;
  int			rc;
  char			errmsg[MAXERRMSG];

  h = (globus_rls_handle_t *) handle;
  lfn = (char *) (*env)->GetStringUTFChars(env, jlfn, NULL);
  lrc = (char *) (*env)->GetStringUTFChars(env, jlrc, NULL);
  r = globus_rls_client_rli_mapping_exists(h, lfn, lrc);
  (*env)->ReleaseStringUTFChars(env, jlfn, lfn);
  (*env)->ReleaseStringUTFChars(env, jlrc, lrc);
  if (r == GLOBUS_SUCCESS)
    return (jboolean) 1;
  globus_rls_client_error_info(r, &rc, errmsg, MAXERRMSG, GLOBUS_FALSE);
  if (rc != GLOBUS_RLS_MAPPING_NEXIST)
    throwexception1(env, rc, errmsg);
  return (jboolean) 0;
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_rlirlilist(JNIEnv *env, jobject obj,
						 jint handle)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_list_t		*list;
  globus_list_t		*p;
  jclass		lc;
  jmethodID		lcid;
  jmethodID		laid;
  jobject		lo;
  jclass		ic;
  jmethodID		mid;
  jobject		io;
  int			len;
  globus_rls_rli_info_t	*rliinfo;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_rli_rli_list(h, &list);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  /* Create ArrayList object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get ids necessary to create RLIInfo objects	*/
  ic = (*env)->FindClass(env, "org/globus/replica/rls/RLSRLIInfo");
  mid = (*env)->GetMethodID(env, ic, "<init>", "(Ljava/lang/String;III)V");

  for (p = list; p; p = globus_list_rest(p)) {
    rliinfo = (globus_rls_rli_info_t *) globus_list_first(p);
    io = (*env)->NewObject(env, ic, mid,
			   (*env)->NewStringUTF(env, rliinfo->url),
			   rliinfo->updateinterval, rliinfo->flags,
			   rliinfo->lastupdate);
    (*env)->CallBooleanMethod(env, lo, laid, io);
  }
  globus_rls_client_free_list(list);
  return lo;
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_getconfiguration(JNIEnv *env,
	jobject obj, jint handle, jstring joption)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*option;
  globus_list_t		*list;

  h = (globus_rls_handle_t *) handle;
  if (joption)
    option = (char *) (*env)->GetStringUTFChars(env, joption, NULL);
  else
    option = NULL;
  r = globus_rls_client_get_configuration(h, option, &list);
  if (option)
    (*env)->ReleaseStringUTFChars(env, joption, option);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  return makestring2list(env, list);
}

JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_setconfiguration(JNIEnv *env,
	jobject obj, jint handle, jstring joption, jstring jvalue)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  char			*option;
  char			*value;

  h = (globus_rls_handle_t *) handle;
  option = (char *) (*env)->GetStringUTFChars(env, joption, NULL);
  value = (char *) (*env)->GetStringUTFChars(env, jvalue, NULL);
  r = globus_rls_client_set_configuration(h, option, value);
  (*env)->ReleaseStringUTFChars(env, joption, option);
  (*env)->ReleaseStringUTFChars(env, jvalue, value);
  if (r != GLOBUS_SUCCESS)
    throwexception(env, r);

  return GLOBUS_RLS_SUCCESS;
}

JNIEXPORT jobject JNICALL
Java_org_globus_replica_rls_RLSClient_stats(JNIEnv *env,jobject obj,jint handle)

{
  globus_result_t	r;
  globus_rls_handle_t	*h;
  globus_rls_stats_t	rlsstats;
  jclass		c;
  jmethodID		mid;

  h = (globus_rls_handle_t *) handle;
  r = globus_rls_client_stats(h, &rlsstats);
  if (r != GLOBUS_SUCCESS) {
    throwexception(env, r);
    return NULL;
  }

  c = (*env)->FindClass(env, "org/globus/replica/rls/RLSStats");
  mid = (*env)->GetMethodID(env, c, "<init>","(Ljava/lang/String;IIIIIIIIIII)V");

  return (*env)->NewObject(env, c, mid,
			   (*env)->NewStringUTF(env, rlsstats.version),
			   rlsstats.uptime, rlsstats.flags,
			   rlsstats.lrc_bloomfilterui, rlsstats.lrc_lfnlistui,
			   rlsstats.lrc_numlfn, rlsstats.lrc_numpfn,
			   rlsstats.lrc_nummap, rlsstats.rli_numlfn,
			   rlsstats.rli_numlrc, rlsstats.rli_numsender,
			   rlsstats.rli_nummap);
}

/*
 * This is a hack to allow making calls to globus-url-copy from RLSGui.
 * Doesn't really belong here.
 */
JNIEXPORT jint JNICALL
Java_org_globus_replica_rls_RLSClient_system(JNIEnv *env, jobject obj, jstring jcmd)

{
  char	*cmd;
  int	r;

  cmd = (char *) (*env)->GetStringUTFChars(env, jcmd, NULL);
  r = system(cmd);
  (*env)->ReleaseStringUTFChars(env, jcmd, cmd);
  return r;
}

static jobject
makeattrlist(JNIEnv * env, globus_list_t *list, int v)

{
  globus_list_t			*p;
  jclass			lc;
  jmethodID			lcid;
  jmethodID			laid;
  jobject			lo;
  jclass			ac;
  jobject			ao;
  globus_rls_attribute_t	*a;
  int				len;

  /* Create List object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get class/method needed to make RLSAttribute objects	*/
  ac = (*env)->FindClass(env, "org/globus/replica/rls/RLSAttribute");

  for (p = list; p; p = globus_list_rest(p)) {
    a = (globus_rls_attribute_t *) globus_list_first(p);
    ao = attr2jattr(env, a, v, ac);
    (*env)->CallBooleanMethod(env, lo, laid, ao);
  }
  globus_rls_client_free_list(list);
  return lo;
}

static jobject
makeattrobjlist(JNIEnv * env, globus_list_t *list)

{
  globus_list_t			*p;
  jclass			lc;
  jmethodID			lcid;
  jmethodID			laid;
  jobject			lo;
  jclass			attrc;
  jobject			attro;
  jclass			attrobjc;
  jobject			attrobjo;
  jmethodID			aoid;
  globus_rls_attribute_object_t	*ao;
  int				len;

  /* Create List object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get class/method needed to make RLSAttributeObject objects	*/
  attrc = (*env)->FindClass(env, "org/globus/replica/rls/RLSAttribute");
  attrobjc = (*env)->FindClass(env, "org/globus/replica/rls/RLSAttributeObject");
  aoid = (*env)->GetMethodID(env, attrobjc, "<init>",
			     "(ILorg/globus/replica/rls/RLSAttribute;Ljava/lang/String;)V");

  for (p = list; p; p = globus_list_rest(p)) {
    ao = (globus_rls_attribute_object_t *) globus_list_first(p);
    attro = attr2jattr(env, &ao->attr, 1, attrc);
    attrobjo = (*env)->NewObject(env, attrobjc, aoid, ao->rc, attro,
				 (*env)->NewStringUTF(env, ao->key));
    (*env)->CallBooleanMethod(env, lo, laid, attrobjo);
  }
  globus_rls_client_free_list(list);
  return lo;
}

static jobject
makestringlist(JNIEnv *env, globus_list_t *list)

{
  globus_list_t		*p;
  jclass		lc;
  jmethodID		lcid;
  jmethodID		laid;
  jobject		lo;
  char			*s;
  jobject		so;
  int			len;

  /* Create List object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  for (p = list; p; p = globus_list_rest(p)) {
    s = (char *) globus_list_first(p);
    so = (*env)->NewStringUTF(env, s);
    (*env)->CallBooleanMethod(env, lo, laid, so);
  }
  globus_rls_client_free_list(list);
  return lo;
}

static jobject
makestring2list(JNIEnv *env, globus_list_t *list)

{
  globus_list_t		*p;
  jclass		lc;
  jmethodID		lcid;
  jmethodID		laid;
  jobject		lo;
  jclass		s2c;
  jmethodID		s2id;
  jobject		s2o;
  globus_rls_string2_t	*s2;
  int			len;

  /* Create List object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get class/method needed to make RLSString2 objects	*/
  s2c = (*env)->FindClass(env, "org/globus/replica/rls/RLSString2");
  s2id = (*env)->GetMethodID(env, s2c, "<init>",
			     "(Ljava/lang/String;Ljava/lang/String;)V");

  for (p = list; p; p = globus_list_rest(p)) {
    s2 = (globus_rls_string2_t *) globus_list_first(p);
    s2o = (*env)->NewObject(env, s2c, s2id, (*env)->NewStringUTF(env, s2->s1),
			    (*env)->NewStringUTF(env, s2->s2));
    (*env)->CallBooleanMethod(env, lo, laid, s2o);
  }
  globus_rls_client_free_list(list);
  return lo;
}

static jobject
makestring2bulklist(JNIEnv *env, globus_list_t *list)

{
  globus_list_t			*p;
  jclass			lc;
  jmethodID			lcid;
  jmethodID			laid;
  jobject			lo;
  jclass			s2bc;
  jmethodID			s2bid1;
  jmethodID			s2bid2;
  jobject			s2bo;
  globus_rls_string2_bulk_t	*s2b;
  int				len;

  /* Create List object	*/
  lc = (*env)->FindClass(env, "java/util/ArrayList");
  lcid = (*env)->GetMethodID(env, lc, "<init>", "(I)V");
  laid = (*env)->GetMethodID(env, lc, "add", "(Ljava/lang/Object;)Z");
  len = globus_list_len(list);
  lo = (*env)->NewObject(env, lc, lcid, len);

  /* Get class/method needed to make RLSString2Bulk objects	*/
  s2bc = (*env)->FindClass(env, "org/globus/replica/rls/RLSString2Bulk");
  s2bid1 = (*env)->GetMethodID(env, s2bc, "<init>",
			      "(ILjava/lang/String;)V");
  s2bid2 = (*env)->GetMethodID(env, s2bc, "<init>",
			      "(ILjava/lang/String;Ljava/lang/String;)V");

  for (p = list; p; p = globus_list_rest(p)) {
    s2b = (globus_rls_string2_bulk_t *) globus_list_first(p);
    if (s2b->str2.s2)
      s2bo = (*env)->NewObject(env, s2bc, s2bid2, s2b->rc,
			       (*env)->NewStringUTF(env, s2b->str2.s1),
			       (*env)->NewStringUTF(env, s2b->str2.s2));
    else
      s2bo = (*env)->NewObject(env, s2bc, s2bid1, s2b->rc,
			       (*env)->NewStringUTF(env, s2b->str2.s1));
    (*env)->CallBooleanMethod(env, lo, laid, s2bo);
  }
  globus_rls_client_free_list(list);
  return lo;
}

static void
freestrlist(globus_list_t *strlist)

{
  globus_list_t	*p;

  for (p = strlist; p; p = globus_list_rest(p))
    free(globus_list_first(p));
  globus_list_free(strlist);
}

static void
freestr2list(globus_list_t *str2list)

{
  globus_list_t		*p;
  globus_rls_string2_t	*str2;

  for (p = str2list; p; p = globus_list_rest(p)) {
    str2 = (globus_rls_string2_t *) globus_list_first(p);
    if (str2->s1)
      free(str2->s1);
    if (str2->s2)
      free(str2->s2);
    free(str2);
  }
  globus_list_free(str2list);
}

static void
freeattrobjlist(globus_list_t *attrobjlist)

{
  globus_list_t			*p;
  globus_rls_attribute_object_t	*ao;

  for (p = attrobjlist; p; p = globus_list_rest(p)) {
    ao = (globus_rls_attribute_object_t *) globus_list_first(p);
    if (ao->key)
      free(ao->key);
    if (ao->attr.type == globus_rls_attr_type_str)
      if (ao->attr.val.s)
	free(ao->attr.val.s);
    free(ao);
  }
  globus_list_free(attrobjlist);
}

static void
jofflim2offlim(JNIEnv *env, jobject jofflim, jfieldID *ofid,
	       int *offset, int *reslimit)

{
  jclass	ac;
  jfieldID	fid;

  ac = (*env)->FindClass(env, "org/globus/replica/rls/RLSOffsetLimit");
  *ofid = (*env)->GetFieldID(env, ac, "offset", "I");
  *offset = (*env)->GetIntField(env, jofflim, *ofid);
  fid = (*env)->GetFieldID(env, ac, "reslimit", "I");
  *reslimit = (*env)->GetIntField(env, jofflim, fid);
}

static jobject
attr2jattr(JNIEnv *env, globus_rls_attribute_t *a, int v, jclass ac)

{
  jmethodID	aid;

  if (!v) {
    aid = (*env)->GetMethodID(env, ac, "<init>", "(Ljava/lang/String;II)V");
    return (*env)->NewObject(env, ac, aid, (*env)->NewStringUTF(env, a->name),
			     a->objtype, a->type);
  }
  switch (a->type) {
    case globus_rls_attr_type_date:
      aid = (*env)->GetMethodID(env, ac, "<init>",
			    "(Ljava/lang/String;III)V");
      return (*env)->NewObject(env, ac, aid, (*env)->NewStringUTF(env,a->name),
			       a->objtype, a->type, a->val.t);
    case globus_rls_attr_type_flt:
      aid = (*env)->GetMethodID(env, ac, "<init>",
			    "(Ljava/lang/String;ID)V");
      return (*env)->NewObject(env, ac, aid, (*env)->NewStringUTF(env,a->name),
			       a->objtype, a->val.d);
    case globus_rls_attr_type_int:
      aid = (*env)->GetMethodID(env, ac, "<init>",
			    "(Ljava/lang/String;III)V");
      return (*env)->NewObject(env, ac, aid, (*env)->NewStringUTF(env,a->name),
			       a->objtype, a->type, a->val.i);
    case globus_rls_attr_type_str:
      aid = (*env)->GetMethodID(env, ac, "<init>",
			    "(Ljava/lang/String;ILjava/lang/String;)V");
      return (*env)->NewObject(env, ac, aid, (*env)->NewStringUTF(env,a->name),
			       a->objtype, (*env)->NewStringUTF(env,a->val.s));
  }
  /* Never gets here */
  return NULL;
}

static void
jattr2attr(JNIEnv *env, jobject jattr, globus_rls_attribute_t *a)

{
  jclass	ac;
  jfieldID	fid;
  jmethodID	mid;
  jstring	jname;
  char		*name;
  jstring	jsval;
  char		*sval;

  ac = (*env)->FindClass(env, "org/globus/replica/rls/RLSAttribute");

  fid = (*env)->GetFieldID(env, ac, "name", "Ljava/lang/String;");
  jname = (*env)->GetObjectField(env, jattr, fid);
  name = (char *) (*env)->GetStringUTFChars(env, jname, 0);
  a->name = strdup(name);
  (*env)->ReleaseStringUTFChars(env, jname, name);

  fid = (*env)->GetFieldID(env, ac, "objtype", "I");
  a->objtype = (*env)->GetIntField(env, jattr, fid);

  fid = (*env)->GetFieldID(env, ac, "valtype", "I");
  a->type = (*env)->GetIntField(env, jattr, fid);

  switch (a->type) {
    case globus_rls_attr_type_date:
      mid = (*env)->GetMethodID(env, ac, "GetDateValSeconds", "()I");
      a->val.t = (*env)->CallIntMethod(env, jattr, mid);
      break;
    case globus_rls_attr_type_flt:
      fid = (*env)->GetFieldID(env, ac, "doubleval", "D");
      a->val.d = (*env)->GetDoubleField(env, jattr, fid);
      break;
    case globus_rls_attr_type_int:
      fid = (*env)->GetFieldID(env, ac, "intval", "I");
      a->val.i = (*env)->GetIntField(env, jattr, fid);
      break;
    case globus_rls_attr_type_str:
      fid = (*env)->GetFieldID(env, ac, "strval", "Ljava/lang/String;");
      jsval = (*env)->GetObjectField(env, jattr, fid);
      sval = (char *) (*env)->GetStringUTFChars(env, jsval, 0);
      a->val.s = strdup(sval);
      (*env)->ReleaseStringUTFChars(env, jsval, sval);
      break;
  }
}

static void
throwexception(JNIEnv *env, globus_result_t r)

{
  int			rc;
  jclass		ec;
  jmethodID		cid;
  jobject		e;
  jstring		s;
  char			errmsg[MAXERRMSG];

  globus_rls_client_error_info(r, &rc, errmsg, MAXERRMSG, GLOBUS_FALSE);
  if ((ec = (*env)->FindClass(env, "org/globus/replica/rls/RLSException")) == 0)
    return;
  if ((cid = (*env)->GetMethodID(env, ec, "<init>",
				 "(ILjava/lang/String;)V")) == 0)
    return;
  s = (*env)->NewStringUTF(env, errmsg);
  if ((e = (*env)->NewObject(env, ec, cid, rc, s)) == 0)
    return;
  (*env)->Throw(env, e);
}

static void
throwexception1(JNIEnv *env, int rc, char *errmsg)

{
  jclass		ec;
  jmethodID		cid;
  jobject		e;
  jstring		s;

  if ((ec = (*env)->FindClass(env, "org/globus/replica/rls/RLSException")) == 0)
    return;
  if ((cid = (*env)->GetMethodID(env, ec, "<init>",
				 "(ILjava/lang/String;)V")) == 0)
    return;
  s = (*env)->NewStringUTF(env, errmsg);
  if ((e = (*env)->NewObject(env, ec, cid, rc, s)) == 0)
    return;
  (*env)->Throw(env, e);
}
